Name: openssl
Version: 1.0.2t
Release: 1
License: OpenSSL
Summary: Open Source SSL/TLS toolkit
Url: https://www.openssl.org

BuildRequires: zlib-devel
BuildRequires: pase-build-tools

Source0: https://www.openssl.org/source/openssl-%{version}.tar.gz

# Patch0: config.patch
Patch1: Configure.patch
Patch2: e_os2.h.patch
Patch3: ui_Makefile.patch
Patch4: ui_openssl.c.patch
Patch5: ui_pase.c.patch
Patch6: UPSTREAM_1.1_CVE-2016-2183.patch

%description
OpenSSL is an open source project that provides a robust, commercial-grade, and
full-featured toolkit for the Transport Layer Security (TLS) and Secure Sockets
Layer (SSL) protocols.

It also provides a general-purpose cryptography library.

%package -n libopenssl1_0_0
Summary: OpenSSL 1.0.x library

%description -n libopenssl1_0_0
OpenSSL 1.0.x libraries

%package devel
Summary: OpenSSL 1.0.x development files
Requires: libopenssl1_0_0 >= %{version}

%description devel
OpenSSL 1.0.x development files


%prep
%setup -q

%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1

%build

perl ./Configure \
  pase64-gcc \
  zlib-dynamic \
  no-fips \
  no-idea \
  no-mdc2 \
  no-rc5 \
  no-engines \
  shared \
  threads \
  --prefix=%{_prefix} \
  --openssldir=/QOpenSys/etc/ssl \
  -L%{_libdir} \
  -I%{_includedir} \
  -Wl,-brtl \
  -Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib

# TODO: Make this in to a patch
perl -p -i -e '/LD_LIBRARY_PATH=/d' Makefile.shared

make depend

function gen_symbol_list
{
  # Generate a list of valid symbols to export
  # Because OpenSSL uses the -bexpall linker flag, we must remove symbols pulled in
  # statically from libc, etc that will get re-exported (eg. strcpy, memset, etc)
  #
  # NOTE: We don't remove symbols that start with a dot (ie. .vpaes_cbc_encrypt), but
  # instead we strip the dot. Normally, these functions should be removed, but the
  # the functions written in assembly don't provide the regular symbol (yet somehow
  # they magically get created in the resulting shared library *shrug*) so we'd be
  # missing symbols in our export list
  echo '#.*'
  nm -Bex "$@" | sed -e '/ [^BDT] /d' -e 's/\.//' -e 's/.* [BDT] //' -e '/::/d' | sort | uniq
}

# OpenSSL 1.0.* libcrypto.so depends on fips_premain_dso,
# but the rule doesn't actually build if we're not building
# FIPS support... Since it doesn't get built, that triggers
# libcrypto.so to get rebuilt every time
touch fips_premain_dso

make build_libs
svr4-shrlib libcrypto.so.1.0.0 libcrypto.so.1.0.0 %{_arch}


gen_symbol_list `find crypto -name \*.o` > symbols
ar -X32 x libcrypto.so.1.0.0
grep -xf symbols shr_64.imp > shr_64.imp.new

diff -u shr_64.imp shr_64.imp.new || :

mv shr_64.imp.new shr_64.imp
ar cr libcrypto.so.1.0.0 shr_64.imp
rm shr_64.imp

# Rebuild libssl to link with our fixed libcrypto
rm libssl.so.1.0.0
make libssl.so.1.0.0
svr4-shrlib libssl.so.1.0.0 libssl.so.1.0.0 %{_arch}


gen_symbol_list ssl/*.o > symbols
ar -X32 x libssl.so.1.0.0
grep -xf symbols shr_64.imp > shr_64.imp.new

diff -u shr_64.imp shr_64.imp.new || :

mv shr_64.imp.new shr_64.imp
ar cr libssl.so.1.0.0 shr_64.imp
rm shr_64.imp

# Do a full build now that we've manually fixed libcrypto and libssl
%make_build

perl -p -i -e 's|#!/usr/bin/perl|#!%{_bindir}/perl|' apps/tsget


%install

%make_build INSTALL_PREFIX=%{buildroot} install

rm %{buildroot}/%{_libdir}/*.a
rm -r %{buildroot}/%{_libdir}/engines

mkdir -p %{buildroot}/%{_mandir}
mv %{buildroot}/QOpenSys/etc/ssl/man/* %{buildroot}/%{_mandir}
rmdir %{buildroot}/QOpenSys/etc/ssl/man

mkdir -p %{buildroot}/%{_datadir}/ssl
mv %{buildroot}/QOpenSys/etc/ssl/misc %{buildroot}/%{_datadir}/ssl

# this conflicts with Perl, so remove it
rm %{buildroot}%{_mandir}/man3/threads.3

%files
%defattr(-, qsys, *none)

%{_bindir}/openssl
%{_bindir}/c_rehash

%{_datadir}/ssl
%{_mandir}/man1/*

/QOpenSys/etc/ssl

%files -n libopenssl1_0_0
%defattr(-, qsys, *none)

%{_libdir}/libcrypto.so.*
%{_libdir}/libssl.so.*


%files devel
%defattr(-, qsys, *none)

%{_libdir}/libcrypto.so
%{_libdir}/libssl.so

%{_libdir}/pkgconfig/*

%{_mandir}/man3/*
%{_mandir}/man5/*
%{_mandir}/man7/*

%{_includedir}/openssl

%changelog
* Mon Dec 09 2019 Kevin Adler <kadler@us.ibm.com> - 1.0.2t
- Remove libc functions from import files added due to -bexpall usage
- Clean up descriptions and line wrap to 80 columns
- General cleanups

* Thu Dec 05 2019 Kevin Adler <kadler@us.ibm.com> - 1.0.2t
- Upgrade to 1.0.2t
- Remove conflicting threads man page
- Fixes CVE-2019-1559
- Fixes CVE-2019-1552
- Fixes CVE-2019-1547
- Fixes CVE-2019-1563

* Thu Mar 28 2019 Kevin Adler <kadler@us.ibm.com> - 1.0.2r
- Update to 1.0.2r

* Tue Mar 26 2019 Kevin Adler <kadler@us.ibm.com> - 1.0.2q
- ibm-build-tools -> pase-build-tools

* Thu Dec 13 2018 - Kevin Adler <kadler@us.ibm.com> - 1.0.2q
- Update to 1.0.2q

* Thu Apr 19 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com> - 1.0.2o
- Relax openssl-devel's version requirement on libopenssl1_0_0

* Tue Apr 10 2018 - Kevin Adler <kadler@us.ibm.com> - 1.0.2o
- Update OpenSSL to 1.0.2o

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com> - 1.0.2k
- Add build requirement for zlib
- use perl -pie instead of GNU sed -i

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Use svr4-shrlib from ibm-build-tools instead of private copy

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- rename libopenssl1-devel to openssl-devel
- fix shebang for tsget

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- Use auto dependency resolution instead of requiring pase-libs

* Thu Aug 17 2017 - Kevin Adler <kadler@us.ibm.com>
- Add support to build 64-bit
- Strip LD_LIBRARY_PATH from Makefile

* Mon Jun 05 2017 - Kevin Adler <kadler@us.ibm.com>
- Use make_build and make_install macros

* Thu Apr 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Add specs
